function [Result RRFinal activemiRNA] = ConditionmiRActivity(data1,data2,miRNA,mRNA,Targetbinding)
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  %%%%%%%%%%%%%
%% Two Steps to infer condition-specific miRNA activity %%%%%%%%%%%%%%%%%%%
%% Step 1: Evaluating all miRNA-mRNA interactions of different correlation %%%%%%%%%%
%% methods using KS test and further combine target binding information %%%%%%%%%%%%
%% MicroCosm to identify significant miRNA-mRNA interactions %%%%%%%%%%
%% Step 2: Inferring actitity of each miRNA with KS test %%%%%%%%%%
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%% Step 1: Evaluating all miRNA-mRNA interactions of different correlation methods
%% using KS test and further combine target binding information MicroCosm.
%% Created by Junpeng Zhang
%% October 2013
%% Input: data1, data2, miRNA, mRNA and MicroCosm.
%% data1 is the bootstrapping result of E samples from R, and data2 is the
%% bootstrapping result of M samples from R, miRNA is the gene names of miRNAs, mRNA
%% is the gene names of mRNAs. MicroCosm is putative target binding
%% information
%% Output: Result and RRFinal.
%% Result is all miRNA-mRNA interactions, and Final is significant miRNA-mRNA interactions with adjusted p-value under KS test. The
%% smaller the adjusted p-value is, the more significant the interaction is.
[m n]=size(data1);
nbootstr=100; %The number of bootstrapping
n=n/nbootstr;

% Computing the median value of each miRNA-mRNA interaction, and make KS
% test for each interaction
for i=1:n
    for j=1:m
        Result{(i-1)*m+j,1}=miRNA{i,1};
        Result{(i-1)*m+j,2}=mRNA{j,1};
        Result{(i-1)*m+j,3}=median(data1(j,i:n:n*nbootstr));
        Result{(i-1)*m+j,4}=median(data2(j,i:n:n*nbootstr));
       [Result{(i-1)*m+j,5},Result{(i-1)*m+j,6},Result{(i-1)*m+j,7}]=kstest2(data1(j,i:n:n*nbootstr),data2(j,i:n:n*nbootstr));
    end
end

% Extracting p-value from the Result, and convert it to numerical data type
for k=1:m*n             
    pvalue{k,1}=Result{k,6};
end

pvalue=cell2mat(pvalue);
adjpvalue=mafdr(pvalue, 'BHFDR', true); %% adjustment of p-value, the method is BH.

% Adding adjusted p-value to the Result, and convert it to cell data type
for q=1:m*n             
    Result{q,8}=adjpvalue(q,1);
end

[m1 n1]=size(Targetbinding);
for p1=1:m*n
    for p2=1:m1
        if(strcmp(Result{p1,1},Targetbinding{p2,1})==1 && strcmp(Result{p1,2},Targetbinding{p2,2})==1)
            Result{p1,9}=1;
        end
    end
end

% Extracting significant miRNA-mRNA intreractions with target binding
% information
index=0;
for p3=1:m*n
    if(Result{p3,8}<0.05 && isempty(Result{p3,9})==0)
        index=index+1;
        Final{index,1}=Result{p3,1};
        Final{index,2}=Result{p3,2};
        Final{index,3}=Result{p3,3};
        Final{index,4}=Result{p3,4};
        Final{index,5}=Result{p3,5};
        Final{index,6}=Result{p3,6};
        Final{index,7}=Result{p3,7};
        Final{index,8}=Result{p3,8};
    end
end


%% Removing identical significant miRNA-mRNA interactions
RFinal=sortrows(Final,8);
for q1=1:index-1
    for q2=q1+1:index
        if(strcmp(RFinal{q1,1},RFinal{q2,1})==1 && strcmp(RFinal{q1,2},RFinal{q2,2})==1)
            RFinal{q2,1}=[];
            RFinal{q2,2}=[];
        end
    end
end

Rindex=0;
for q3=1:index
    if(isempty(RFinal{q3,1})==0)
        Rindex=Rindex+1;
        RRFinal{Rindex,1}=RFinal{q3,1};
        RRFinal{Rindex,2}=RFinal{q3,2};
        RRFinal{Rindex,3}=RFinal{q3,3};
        RRFinal{Rindex,4}=RFinal{q3,4};
        RRFinal{Rindex,5}=RFinal{q3,5};
        RRFinal{Rindex,6}=RFinal{q3,6};
        RRFinal{Rindex,7}=RFinal{q3,7};
        RRFinal{Rindex,8}=RFinal{q3,8};
    end
end

RRFinal=sortrows(RRFinal,1);

%Save the results of all interactions and significant miRNA-mRNA interactions with target binding information under KS test
%xlswrite('All-miRNA-mRNA-kstest.xlsx',Result);
%xlswrite('Significant-miRNA-mRNA-kstest.xlsx',RRFinal);


for p4=1:Rindex
     miRNAstring{p4,1}=RRFinal{p4,1};
     data{p4,1}=RRFinal{p4,3};
     data{p4,2}=RRFinal{p4,4};
end
data=cell2mat(data);
%% Step 2: Inferring actitity of each miRNA
%% Input: miRNAstring, data
%% miRNAstring is ascending or descending order of miRNA, and data is the
%% median value of each significant miRNA-mRNA interaction with target information
%% Output: activemiRNA
%% activemiRNA is the activity of each miRNA with p-value and adjusted p-value
%% under KS test.
activemiRNA=tabulate(miRNAstring); % miRNAstring is ascending or descending order of miRNA in significant miRNA-mRNA interaction with target information
[M N]=size(activemiRNA);

% Extracting the number of interactions for each miRNA
for j=1:M
    K{j,1}=activemiRNA{j,2};
end
K=cell2mat(K);

% Calculating the activity of each miRNA with KS test
for i=1:M
    [h(i,1),Pvalue(i,1),r(i,1)]=kstest2(data(sum(K(1:i-1,1))+1:sum(K(1:i,1)),1),data(sum(K(1:i-1,1))+1:sum(K(1:i,1)),2));
end
adjPvalue=mafdr(Pvalue, 'BHFDR', true);

%Combine the p-value and adjusted p-value of each miRNA into the
%activemiRNA
for KK=1:M
    activemiRNA{KK,2}=Pvalue(KK,1);
    activemiRNA{KK,3}=adjPvalue(KK,1);
end

activemiRNA=sortrows(activemiRNA,3);
%Save the results of all miRNA activity under KS test
%xlswrite('miRNA activity.xlsx',activemiRNA);

% Generate the header of Result, RRFinal and activemiRNA
Header1{1,1}='miRNA';Header1{1,2}='mRNA';Header1{1,3}='Value1';Header1{1,4}='Value2';Header1{1,5}='h-value';Header1{1,6}='p-value';Header1{1,7}='ks2stat';Header1{1,8}='adjp-value';Header1{1,9}='Overlap';
Header2{1,1}='miRNA';Header2{1,2}='mRNA';Header2{1,3}='Value1';Header2{1,4}='Value2';Header2{1,5}='h-value';Header2{1,6}='p-value';Header2{1,7}='ks2stat';Header2{1,8}='adjp-value';
Header3{1,1}='miRNA';Header3{1,2}='p-value';Header3{1,3}='adjp-value';

Result=[Header1;Result];
RRFinal=[Header2;RRFinal];
activemiRNA=[Header3;activemiRNA];

