# Generating all miRNA-mRNA interactions of different correlation methods by sample splitting with Bootstrapping
# Created by Junpeng Zhang
# October 2013

#Read the dataset with .csv format, the first column should be the #rownames (sample names), the first row is the column names (gene #names).
#param: filename (.csv)
#return: data table
	Read<-function(dataset){
		data<-read.csv(dataset, header=TRUE, sep=",")
		return(data)
	}

#Standardise the dataset to have unit standard deviation.
#param: data table
#return: standardised data set
	Standardise<-function(dataset){
		ncol<-ncol(dataset)
		nrow<-nrow(dataset)
		stdData<-matrix(nrow=nrow,ncol=ncol-1)
                rownames(stdData)<-dataset[,1]
                colnames<-colnames(dataset)
                colnames(stdData)<-colnames[2:ncol]
		for (i in 2:ncol){
			stdData[,i-1]<-scale(dataset[i],center=TRUE, scale=TRUE)
			}
		return(stdData)
	}

#Calculate the rowmedian of a matrix
    rowmedian <- function(Matrix){
          row <- nrow(Matrix)
          medianresult<-matrix(nrow=row,ncol=1)
          for (i in 1:row){
                 medianresult[i,1] <- median(Matrix[i,])
}
         return(medianresult)
}




#1. IDA
#Learning PCDAG from data then inferring causal effects
#required: pcalg package
#param: standardised dataset, turning parameter for PC algorithm, number of nodes need to
#intervene (number of miRNAs)
#return: PCDAG
	IDA<-function(stdData,alp, nommiRs){
		library(pcalg)
	#Learning PCDAG
		suffStat<-list(C=cor(stdData), n=nrow(stdData))  
		pc.fit<-pc(suffStat, indepTest=gaussCItest, p=ncol(stdData), alpha=alp)	
		nommRs<-ncol(stdData)-nommiRs
		result<-matrix(nrow=nommRs, ncol=nommiRs)
		for (l in 1:nommiRs){
				
			#Inferring causal effects
			caef<-idaFast(l,(nommiRs+1):ncol(stdData),cov(stdData), pc.fit@graph )
		
			#min of absolute values.
			caef1<-matrix(nrow=nommRs,ncol=1)
			for (k in 1:nommRs){
				caefabs<-abs(caef)
				index<-which(caefabs==min(caefabs[k,]), arr.ind=TRUE)
				pos<-index[1,2]
				caef1[k,]<-caef[k,pos]
			}
			result[,l]<-caef1
		}
		temp<-"Finish!"
		print(temp)
		return(result)
	}


# Bootstrapping of IDA
bootstrIDA<-function(stdData, nbootstr, alp, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) IDA(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], alp, nommiRs))
	 
         return(Result) # Return bootstrapping results
                
        	
}

#2.Pearson correlation
Pearson <- function(stdData, nommiRs) {
        # Pearson correlation of miRNA and mRNA within the data matrix
        col <- ncol(stdData)
        rMatrix <- cor(stdData[,(nommiRs+1):col], stdData[,1:nommiRs], use="pairwise", method="pearson") # Calculate Pearson correlation
        return(rMatrix)
}

# Bootstrapping of Pearson
bootstrPearson<-function(stdData, nbootstr, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) Pearson(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], nommiRs))
	 
         return(Result) # Return bootstrapping results
        	
}


#3. Spearman correlation
Spearman <- function(stdData, nommiRs) {
        # Spearman correlation of miRNA and mRNA within the data matrix
        col <- ncol(stdData)
        rMatrix <- cor(stdData[,(nommiRs+1):col], stdData[,1:nommiRs], use="pairwise", method="spearman") # Calculate Spearman correlation
        return(rMatrix)
}

# Bootstrapping of Spearman
bootstrSpearman<-function(stdData, nbootstr, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) Spearman(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], nommiRs))
	 
         return(Result) # Return bootstrapping results
        	
}

#4. Kendall correlation
Kendall <- function(stdData, nommiRs) {
        # Kendall correlation of miRNA and mRNA within the data matrix
        col <- ncol(stdData)
        rMatrix <- cor(stdData[,(nommiRs+1):col], stdData[,1:nommiRs], use="pairwise", method="kendall") # Calculate Kendall correlation
        return(rMatrix)
}

# Bootstrapping of Kendall
bootstrKendall<-function(stdData, nbootstr, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) Kendall(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], nommiRs))
	 
         return(Result) # Return bootstrapping results
        	
}



#5. Lasso
Lasso <- function(stdData, nommiRs) {
        # Lasso coefficient of miRNA and mRNA within the data matrix
        library(glmnet)
        col <- ncol(stdData)
        rMatrix <- matrix(nrow=nommiRs, ncol=col-nommiRs)
        for(i in (nommiRs+1):col) {
                Matrix <- glmnet(stdData[,1:nommiRs],stdData[,i],alpha=1)$beta  # Calculate Lasso correlation
                col <- ncol(Matrix)
                Matrix_1 <- matrix(Matrix,nommiRs,col)
                rMatrix[,i-nommiRs] <- rowmedian(Matrix_1)                 
}
                return(rMatrix)
}

# Bootstrapping of Lasso
bootstrLasso <- function(stdData, nbootstr, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) Lasso(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], nommiRs))
	 
         return(Result) # Return bootstrapping results
        	
}


#6. Elastic-net
Elastic <- function(stdData, nommiRs) {
        # Elastic coefficient of miRNA and mRNA within the data matrix
        library(glmnet)
        col <- ncol(stdData)
        rMatrix <- matrix(nrow=nommiRs, ncol=col-nommiRs)
        for(i in (nommiRs+1):col) {
                Matrix <- glmnet(stdData[,1:nommiRs],stdData[,i],alpha=0.5)$beta  # Calculate Elastic correlation
                col <- ncol(Matrix)
                Matrix_1 <- matrix(Matrix,nommiRs,col)
                rMatrix[,i-nommiRs] <- rowmedian(Matrix_1)                 
}
                return(rMatrix)
}

# Bootstrapping of Elastic
bootstrElastic <- function(stdData, nbootstr, nommiRs){
	 Result<-lapply(1:nbootstr, function(i) Elastic(stdData[sample(nrow(stdData), nrow(stdData), replace=T), ], nommiRs))
	 
         return(Result) # Return bootstrapping results
        	
}











        